# encoding: UTF-8

#
#  utex/filter.rb  --  Convert from and to TeX specific encoding
#

require "supplement"

module UTeX

  MAPS = {}
  PATTERNS = {}
  HYPHENATIONS = {}

  class Encoder

    class Error < StandardError ; end

    attr_reader :enc

    def initialize enc = nil
      @enc = enc||:cmr
      require "utex/maps/#@enc"
    end

    SPECIALS = {}
    %Q(\\{}$&\r\n#^_\0\t \f~%\x7f`).each_codepoint { |c| SPECIALS[ c] = true }

    def encode l
      r = ""
      l.each_codepoint { |c|
        unless SPECIALS[ c] then
          s = UTeX::MAPS[ @enc][ c]
          s ||= (c if (0x21..0x7e).cover? c)
          s or raise Error, "U%04x (%s) has no representation in %s." %
                                                [ c, (c.chr l.encoding), @enc]
          c = s
        end
        r << c.chr
      }
      r
    end


    def catcodes out = nil
      require "utex/uclc"
      out ||= ""
      out << "% Catcodes of #@enc" << $/
      map = UTeX::MAPS[ @enc]
      map.each { |up,cp|
        next if cp.chr =~ /[a-z]/i
        upu, upl = UC[ up], LC[ up]
        next unless upu or upl
        out << %q{\catcode"%02X=11} % cp
        out << %q{  \lccode"%02X="%02X} % [cp, map[ upl]||cp]
        out << %q{  \uccode"%02X="%02X} % [cp, map[ upu]||cp]
        out << %q{  %% U%04x -> %s} % [ up, cp.chr]
        out << $/
      }
      out
    end

  end


  class Filter

    class <<self

      attr_accessor :keep

      def build utex
        tex = utex.sub /(\.utex)?$/, ".tex"
        File.open utex do |uf|
          File.open tex, "w" do |tf|
            f = new tf
            f.do_file uf
          end
        end
        if block_given? then
          begin
            yield tex
          ensure
            File.unlink tex unless @keep
          end
        else
          tex
        end
      end

      def run *args
        build args.pop do |tex|
          cmd = []
          cmd.push "tex"
          cmd.concat args
          cmd.push tex
          system *cmd
        end
      end

    end

    def initialize out
      @out = out
      @encs = []
    end

    def do_file file
      file.each_line { |l|
        putline l
      }
    end

    def putline l
      if l =~ /^\s*%%\s*UTeX:\s*(\w+)\s*(.*?)\s*$/i then
        cmd, param = $~.captures
        case cmd
          when "encoding", "enc" then
            case param
              when "end"       then @encs.pop
              when "none", "-" then @encs.push nil
              else
                e = Encoder.new param.to_sym
                @encs.push e
            end
          when "catcodes", "cat" then
            @encs.last.catcodes @out
          when "patterns", "pat" then
            require "utex/patterns/#{param}"
            put UTeX::PATTERNS[ param.to_sym]
          when "hyphenations", "pat" then
            require "utex/patterns/#{param}"
            put UTeX::HYPHENATIONS[ param.to_sym]
          else
            raise "Unknown UTeX command: #$1 #$2"
        end
      else
        put l
      end
    rescue
      $stderr.puts l
      raise
    end

    private

    def put l
      l.notempty? or return
      e = @encs.last
      l = e.encode l if e
      @out.puts l
    end

  end

end

