#
#  lib/utex/suite.rb  --  UTeX-TeX-dvips-ps2pdf-Suite
#

require "utex/filter"
require "etc"


module UTeX

  class Suite

    u = Etc.getpwuid( Process.uid).name
    TMP = File.expand_path "utex.#{u}", "/tmp"

    SUFFIX = "-%Y%m%d-%H%M%S"

    class <<self
      private :new
      def open *args
        File.directory? TMP or Dir.mkdir TMP
        Dir.chdir TMP do
          i = new *args
          begin
            yield i
          ensure
            i.cleanup
          end
        end
      end
    end

    def initialize input, generator = nil
      @input, @generator = input, generator
      @name = case @input
        when File then
          b = File.basename @input.path
          if b =~ /\A(.+)\.\w+\z/ then $1 else b end
        else
          "job"
      end
      @name << "#{Time.now.strftime SUFFIX}-#$$"
      @built = {}
    end

    def keep b
      @built.delete b
    end

    def keep_all
      @built.clear
    end

    def cleanup
      @built.each { |t,b| File.unlink b rescue nil }
    end

    def read_utex
      if @generator then
        utex = "#@name.utex"
        File.open utex, "w" do |uf|
          g = @generator.new uf
          g.run @input
        end
        notify :utex, utex
        File.open utex do |uf|
          yield uf
        end
      else
        yield @input
      end
    end
    private :read_utex

    def mktex &block
      tex, log = "#@name.tex", "#@name.log"
      read_utex do |utex|
        File.open tex, "w" do |tf|
          f = UTeX::Filter.new tf
          f.do_file utex
        end
      end
      notify :log, log, &block
      notify :tex, tex, &block
    end

    def mkdvi &block
      tex = mktex &block
      dvi = "#@name.dvi"
      exec_noout "tex", "-parse-first-line", tex
      notify :dvi, dvi, &block
    end

    def mkps &block
      dvi = mkdvi &block
      ps = "#@name.ps"
      exec_noout "dvips", "-R0", dvi
      notify :ps, ps, &block
    end

    def mkpdf &block
      ps = mkps &block
      pdf = "#@name.pdf"
      exec_noout "ps2pdf", ps
      notify :pdf, pdf, &block
    end

    private

    def notify t, f
      @built[ t] = f
      yield t, f if block_given?
      f
    end

    def exec_noout *args
      ri, wi = IO.pipe
      ro, wo = IO.pipe
      re, we = IO.pipe
      child = fork do
        wi.close ; $stdin .reopen ri ; ri.close
        ro.close ; $stdout.reopen wo ; wo.close
        re.close ; $stderr.reopen we ; we.close
        exec *args
      end
      ri.close
      wo.close
      we.close
      wi.close
      ro.read
      re.read
      Process.wait child
    end

  end

end

